/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Some code borrowed from the following extensions: 
 * AdBlock, NukeImage, BugMeNot: THANK YOU!
 *
 * ***** END LICENSE BLOCK ***** */

var Items;
var Weights;
var ItemsTable;
var WeightsTable;
var WeightConstant;
var prefObj;
var Branch;
var tags = ["img", "embed", "object", "iframe"];
var BlockedItems; 
var NonBlockedItems;

// Resolves the absolute URL of the image-address being sent as an argument.
// page is the document which contains the image.
function getImageURL(imageSrc, page) {
	var URL = Components.classes["@mozilla.org/network/standard-url;1"].createInstance(Components.interfaces.nsIURL);	
	URL.spec = page.URL;
	return URL.resolve(imageSrc);	
}

// Checks an element and its children recursively for an attribute.
function findAttribute(element, name) {
	if (element.hasAttribute(name)) {
		return element.getAttribute(name);
	}
	
	if (element.hasChildNodes()) {
		for (var i = 0 ; i < element.childNodes.length ; i++) {
			var res = findAttribute(element.childNodes[i], name);
			if (res != false) 
				return res;
		}
	}
	
	return false; // Didn't find it! 
}

/*
// triggered when right-clicking on an image:
// it should learn this link as a positive sample, to be blocked in the future
// so:
// 1- remove link from list of URLs that are potentially negative samples
// 2- tokenize link (using slashes and dots as delimiters, after removing protocol and file extension of the URL) into items
// 3- learn items positively
// 4- reload if really want to hide the picture right away (but should then be careful not to learn the other pics positively: empty the list first))
*/

function learnMeOnClick(){
  if(gContextMenu) {
    var img = gContextMenu.target;
    if(img) {
      // go away, image!
      img.style.display = "none";
	
	// and never come back!		
	var src = img.getAttribute("src");
	var page = document.getElementById("content").contentDocument;
	var imgUrl = getImageURL(src, page);
	
	//window.alert(" URL= " + imgUrl);
	Items += (" " + imgUrl);
	Weights += (" " + 1);

	//starting Nock's algorithm
	//first update constant 
	WeightConstant = (1/2.73) * WeightConstant;
	
	//create new tokens and init their weights
	tokens = imgUrl.split("/");
	
	for (var i = 0 ; i < tokens.length ; i++) {
		var found = false;
		for (var j = 0; ((j < ItemsTable.length) && (found == false)) ; j++) {
			if (tokens[i] == ItemsTable[j]) {
				found = true;
			}
		}
		//only add tokens that aren't already in the items list
		if (found == false) {
			if (tokens[i] != "") {
				//empty token happens in splitting http://...
				Items += (" " + tokens[i]);
				Weights += (" " + 1);
			}
		}

	}

    }
  }
  
}

function unlearnMeOnClick(){
	//alert("Blocked Items: " + BlockedItems);
	var page = document.getElementById("content").contentDocument;
	var BlockedItemTable = BlockedItems.split(" ");
	var selection = {};
	var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                                    .getService(Components.interfaces.nsIPromptService);
	promptService.select(window, "Unblock image", "Select the image that you want to unblock:",
                           BlockedItemTable.length, BlockedItemTable, selection);
      //alert("You selected: " + BlockedItemTable[selection.value]);

	var found = false;
	for (var j = 0 ; ((j < tags.length) && (found == false)) ; j++) {
		
		var scanTags = page.getElementsByTagName(tags[j]);
		for (var i = 0 ; i < scanTags.length ; i++) {

			var src = findAttribute(scanTags[i],"src");
			var srcurl = getImageURL(src, page);
			if (src && (srcurl == BlockedItemTable[selection.value]) ) {
				//alert("Found: " + scanTags[i]);
				found = true;
				scanTags[i].style.display = "block";
			}
		}
	}


}

// triggered when browser starts
// read items + weights from system properties
// FOR NOW: CALLED ON LOAD

function readOnStart() {
		try {
		prefObj = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
		Branch = prefObj.getBranch("extensions.adblocklearner.");
		Items = Branch.getCharPref("items");
		Weights = Branch.getCharPref("weights");
		WeightConstant = Branch.getCharPref("constant");
		
	} catch(exception) {
		//this happens when the parameters are empty
		//ugly hack to start with something		
		//alert("Exception! " + ItemsTable);
		Items = "http:";
		Weights = "0";
		WeightConstant = "1";
	}
	//alert("Read: " + ItemsTable);
	ItemsTable = Items.split(" ");
	WeightsTable = Weights.split(" ");
}

// triggered when browser quits
// store updated items + weights back into system properties

function writeOnQuit () {
	//window.alert("Saving prefs!");
	Branch.setCharPref("items", Items);
	Branch.setCharPref("weights", Weights);
	Branch.setCharPref("constant", WeightConstant);
}

// This method checks if a picture matches a filter.
// It returns true if the imageurl is caught by one of the filters.
// It returns false if the imageurl is not caught by any filter.
function validate(imageurl) {
	//alert("Validating " + imageurl);
	var sum = 0;
	for (var i = 0 ; i < ItemsTable.length ; i++) {
		if ( imageurl.indexOf(ItemsTable[i]) != -1) {
			//alert(ItemsTable[i] + " matches " + imageurl);
			sum += (1 * WeightsTable[i]);
		}
	}
	if (sum >= WeightConstant) return true;	
	return false;
}


// triggered when page is loading
// 1- block ads that are over threshold 
// 2- put all other pic URLs in item list
// also: register to right-click popup menu!

function filterOnLoad () {
	document.getElementById("contentAreaContextMenu").addEventListener("popupshowing",hideLearnerOffContext,false);
	readOnStart();
	
	var page = document.getElementById("content").contentDocument;
	BlockedItems = "";
	NonBlockedItems = "";

	for (var j = 0 ; j < tags.length ; j++) {
		
		var scanTags = page.getElementsByTagName(tags[j]);
		for (var i = 0 ; i < scanTags.length ; i++) {

			var src = findAttribute(scanTags[i],"src");
			var srcurl = getImageURL(src, page);
			if (src && validate(srcurl)) {
				//alert("Checking: " + scanTags[i]);
				scanTags[i].style.display = "none";
				BlockedItems += (srcurl + " ");
			}
			else {
				NonBlockedItems += (srcurl + " ");
			};
		}
	}
}

function hideLearnerOffContext () {
  document.getElementById("adblocklearner").hidden = !gContextMenu.onImage;
}


// triggered when leaving a page
// 1- tokenize all URLs left in list
// 2- learn items negatively (were not nuked by user)
// FOR NOW: UPDATING ITEMS HERE

function learnRestOnLeave () {
      writeOnQuit();
}

window.addEventListener("load",filterOnLoad,true);
window.addEventListener("unload",learnRestOnLeave,true);


